package;

import flash.display.InteractiveObject;
import flash.display.Sprite;
import flixel.FlxG;
import flixel.FlxObject;
import flixel.FlxSprite;
import flixel.FlxState;
import flixel.group.FlxGroup.FlxTypedGroup;
import flixel.math.FlxPoint;
import flixel.tile.FlxTile;
import flixel.tile.FlxTilemap;
import flixel.text.FlxText;
import flixel.ui.FlxButton;
import flixel.math.FlxMath;
import flixel.math.FlxRandom;
import seeds.GreenSeed;
import seeds.OrangeSeed;
import seeds.YellowSeed;
import tiles.Green;
import tiles.Yellow;
import tiles.Purple;
import tiles.Red;
import tiles.Tile;
import seeds.Seed;
import tiles.EmptyTile;

using flixel.util.FlxSpriteUtil;

class PlayState extends FlxState
{
	public static inline var GRID_WIDTH:Int = 16;
	public static inline var GRID_HEIGHT:Int = 16;
	public static inline var TILE_SIZE:Float = 8;
	public static inline var TILE_OFFSET_X:Float = 64;
	public static inline var TILE_OFFSET_Y:Float = 12;
	
	public static inline var MOVE_DELAY:Int = 15;
	public static inline var DESTROY_DELAY:Int = 45;
	
	public static inline var SEED_SPAWN_DELAY:Int = 3000;
	public static inline var BERRY_SPAWN_DELAY:Int = 12000;
	public static inline var EVIL_BERRY_SPAWN_DELAY:Int = 24000;
	
	public static inline var EVIL_PLANT_SPAWN_DELAY_START:Int = 750;
	public static inline var EVIL_PLANT_SPAWN_DELAY_DECREASE:Int = 20;
	public static inline var EVIL_PLANT_SPAWN_DELAY_MIN:Int = 240;
	
	public static inline var EVIL_WAVE_DELAY:Int = 3600;
	public static inline var EVIL_WAVE_STARTING_SPAWN_COUNT:Int = 3;
	
	public static inline var EVIL_BERRY_LOSS_COUNT:Int = 5;
	
	public static var playState:PlayState;
	
	public static var state:GameState = MENU;
	public static var tutorialProgress:Int = 0;
	
	public var player:Player;
	
	public var moveDelay:Int;
	
	var nextMove:FlxPoint;
	
	public var gridFeatures:FlxTypedGroup<GridFeature>;
	
	public var grid:Array<Array<Tile>>;
	
	public var seedArray:Array<Seed>;
	
	public var seeds:FlxTypedGroup<Seed>;
	
	public var movementMap:FlxTilemap;
	
	public var tiles:FlxTypedGroup<Tile>;
	
	public var rng:FlxRandom;
	
	public var berryText:FlxText;
	public var evilBerryText:FlxText;
	
	public var seedCount:Int;
	public var seedText:FlxText;
	
	public var berryDelay:Int;
	
	public var evilBerryDelay:Int;
	
	public var evilWaveDelay:Int;
	public var evilWaveSpawnCount:Int;
	public var evilWaveCount:Int;
	
	public var seedDelay:Int;
	
	public var score:Int;
	
	public var gameEndTime:Int;
	public var finalScore:Int;
	
	public var frameCount:Int;
	
	override public function create():Void
	{
		super.create();
		
		rng = new FlxRandom();
		
		frameCount = 0;
		
		seeds = new FlxTypedGroup<Seed>();
		seedArray = new Array<Seed>();
		
		playState = this;
		
		player = new Player(rng.int(Std.int(GRID_WIDTH / 2) - 1, Std.int(GRID_WIDTH / 2)), rng.int(Std.int(GRID_HEIGHT / 2) - 1, Std.int(GRID_HEIGHT / 2)));
		
		moveDelay = 0;
		
		tiles = new FlxTypedGroup<Tile>();
		
		berryDelay = 180;
		evilBerryDelay = EVIL_BERRY_SPAWN_DELAY;
		
		grid = new Array<Array<Tile>>();
		for (i in 0...GRID_WIDTH)
		{
			grid.push(new Array<Tile>());
			for (i2 in 0...GRID_HEIGHT)
			{
				var tile:EmptyTile = new EmptyTile(i, i2);
				tiles.add(tile);
				grid[i].push(tile);
			}
		}
		
		var mapArray:Array<Array<Int>> = new Array<Array<Int>>();
		for (i in 0...GRID_WIDTH)
		{
			mapArray.push(new Array<Int>());
			for (i2 in 0...GRID_HEIGHT)
			{
				mapArray[i].push(0);
			}
		}
		
		movementMap = new FlxTilemap();
		movementMap.loadMapFrom2DArray(mapArray, "assets/images/movementTiles.png", 8, 8, OFF);
		movementMap.setPosition(TILE_OFFSET_X, TILE_OFFSET_Y);
		
		gridFeatures = new FlxTypedGroup<GridFeature>();
		
		berryText = new FlxText(80, 2, 64, "0", 8);
		berryText.alignment = CENTER;
		berryText.color = 0xff0000;
		
		evilBerryText = new FlxText(112, 2, 64, "0", 8);
		evilBerryText.alignment = CENTER;
		evilBerryText.color = 0x880088;
		
		seedCount = 0;
		seedText = new FlxText(Seed.X, 15, 16, "0", 8);
		seedText.alignment = CENTER;
		seedText.color = 0xa08020;
		
		seedDelay = SEED_SPAWN_DELAY;
		
		score = 0;
		
		gameEndTime = -1;
		finalScore = 0;
		
		evilWaveDelay = 600;
		evilWaveCount = 0;
		evilWaveSpawnCount = EVIL_WAVE_STARTING_SPAWN_COUNT;
		
		add(tiles);
		add(movementMap);
		add(gridFeatures);
		add(player);
		add(seeds);
		add(berryText);
		add(evilBerryText);
		add(seedText);
		
		switch(state)
		{
			case PLAY:
				seedCount = 10;
				for (i in 0...4)
				{
					newSeed();
				}
				
				setTile(new Green(player.gridX, player.gridY, 1));
			case TUTORIAL:
				remove(seedText);
				remove(berryText);
				remove(evilBerryText);
				player.setGridPosition(0, 7);
				var goalSquare:FlxSprite = new FlxSprite(TILE_OFFSET_X + TILE_SIZE * (GRID_WIDTH - 2), TILE_OFFSET_Y + 7 * TILE_SIZE, "assets/images/frame.png");
				add(goalSquare);
				switch(tutorialProgress)
				{
					case 0:
						for (i in 0...2)
						{
							setTile(new Green(i, 6, 1, 0));
							setTile(new Green(i, 9, 1, 0));
						}
						var growText:FlxText = new FlxText(0, 12, FlxG.width, "Press Space to grow", 8);
						growText.alignment = CENTER;
						add(growText);
						for (i in 0...4)
						{
							newSeed();
						}
					case 1:
						for (i in 0...2)
						{
							setTile(new Yellow(i, 6, 1, 0));
							setTile(new Yellow(i, 9, 1, 0));
						}
						for (i in 0...4)
						{
							newSeed();
						}
					case 2:
						seedCount = 2;
						for (i in 0...2)
						{
							newSeed();
						}
						add(seedText);
						add(berryText);
						for (i in 0 ... GRID_WIDTH)
						{
							setTile(new Yellow(i, 6, 1, 0));
							setTile(new Green(i, 9, 1, 0));
							gridFeatures.add(new SeedPickup(i, 6));
							var berry:Berry = new Berry(i, 9);
							berry.ripeness = rng.int(0, 3);
							berry.ripenessProgress = rng.int(0, Berry.RIPENESS_PROGRESS_MAX - 1);
							gridFeatures.add(berry);
						}
						var yellowText:FlxText = new FlxText(30, 28, FlxG.width - 60, "Yellow plants grow seeds, which are used to grow new plants.", 8);
						yellowText.alignment = CENTER;
						add(yellowText);
						var greenText:FlxText = new FlxText(30, 100, FlxG.width - 60, "Green plants grow berries. Ripe berries gives points when collected.", 8);
						greenText.alignment = CENTER;
						add(greenText);
					case 3:
						for (i in 0...GRID_WIDTH)
						{
							setTile(new Red(i, 0, 1, rng.int(0, 1)));
							if (rng.float() < 0.25)
							{
								setTile(new Purple(i, GRID_HEIGHT - 1, 1, new FlxPoint(0, 1), rng.int(3,6)));
							}
							else
							{
								setTile(new Purple(i, GRID_HEIGHT - 1, 1, new FlxPoint(0, 0), rng.int(0,3)));
							}
							if (rng.float() < 0.25)
							{
								var b:EvilBerry = new EvilBerry(i, 0);
								b.ripeness = rng.int(0, 3);
								b.ripenessProgress = rng.int(0, EvilBerry.RIPENESS_PROGRESS_MAX);
								gridFeatures.add(b);
							}
							if (rng.float() < 0.25)
							{
								var b:EvilBerry = new EvilBerry(i, GRID_HEIGHT - 1);
								b.ripeness = rng.int(0, 3);
								b.ripenessProgress = rng.int(0, EvilBerry.RIPENESS_PROGRESS_MAX);
								gridFeatures.add(b);
							}
							setTile(new Green(i, 6, 1, 0));
							setTile(new Green(i, 9, 1, 0));
						}
						
						for (i in 0...4)
						{
							setTile(new Green(i * 4, 2, 1));
						}
						
						var evilText:FlxText = new FlxText(30, 102, FlxG.width - 60, "Evil plants grow evil berries. When five evil berries ripen, the game ends.");
						evilText.alignment = CENTER;
						add(evilText);
				}
			case MENU:
				remove(seeds);
				remove(seedText);
				remove(berryText);
				remove(evilBerryText);
				remove(tiles);
				remove(movementMap);
				
				player.setGridPosition(player.gridX, 10);
				
				var titleText:FlxText = new FlxText(0, 16, FlxG.width, "Brazen Berry Bonanza", 16);
				titleText.alignment = CENTER;
				add(titleText);
				var playText:FlxText = new FlxText(TILE_OFFSET_X + TILE_SIZE - 64, 64, 128, "Play", 12);
				playText.alignment = CENTER;
				add(playText);
				var tutorialText:FlxText = new FlxText(TILE_OFFSET_X + TILE_SIZE * (GRID_WIDTH - 1) - 64, 64, 128, "Tutorial", 12);
				tutorialText.alignment = CENTER;
				add(tutorialText);
				
				var playSquare:FlxSprite = new FlxSprite(TILE_OFFSET_X, TILE_OFFSET_Y + 10 * TILE_SIZE, "assets/images/frame.png");
				add(playSquare);
				var tutorialSquare:FlxSprite = new FlxSprite(TILE_OFFSET_X + TILE_SIZE * (GRID_WIDTH - 2), TILE_OFFSET_Y + 10 * TILE_SIZE, "assets/images/frame.png");
				add(tutorialSquare);
				
				var movementText = new FlxText(0, 120, FlxG.width, "WASD or Arrow keys to move", 8);
				movementText.alignment = CENTER;
				add(movementText);
		}
	}

	override public function update(elapsed:Float):Void
	{
		super.update(elapsed);
		
		switch(state)
		{
			case PLAY:
				frameCount++;
				evilWaveDelay--;
				if (evilWaveDelay <= 0)
				{
					evilWaveDelay = EVIL_WAVE_DELAY;
					for (i in 0...EVIL_WAVE_STARTING_SPAWN_COUNT + evilWaveCount)
					{
						spawnEvilPlant();
					}
					evilWaveCount++;
				}
				
				if (moveDelay > 0)
				{
					moveDelay--;
				}
				if (moveDelay <= 0)
				{
					if (FlxG.keys.anyPressed(["W", "w", UP]))
					{
						nextMove = new FlxPoint(0, -1);
					}
					if (FlxG.keys.anyPressed(["S", "s", DOWN]))
					{
						nextMove = new FlxPoint(0, 1);
					}
					if (FlxG.keys.anyPressed(["A", "a", LEFT]))
					{
						nextMove = new FlxPoint(-1, 0);
					}
					if (FlxG.keys.anyPressed(["D", "d", RIGHT]))
					{
						nextMove = new FlxPoint(1, 0);
					}
					if (nextMove != null)
					{
						var moveX:Int = Std.int(nextMove.x);
						var moveY:Int = Std.int(nextMove.y);
						var newX:Int = player.gridX + moveX;
						var newY:Int = player.gridY + moveY;
						if (newX >= 0 && newY >= 0 && newX < GRID_WIDTH && newY < GRID_HEIGHT && (movementMap.getTile(newX, newY) == 1 || getTile(newX, newY).walkable))
						{
							player.setGridPosition(newX, newY);
							moveDelay = MOVE_DELAY;
							for (g in gridFeatures)
							{
								if (Std.is(g, Berry))
								{
									var b:Berry = cast(g, Berry);
									if (b.ripeness == 3 && b.gridX == player.gridX && b.gridY == player.gridY)
									{
										gridFeatures.remove(b);
										score++;
									}
								}
							}
							if (!Std.is(getTile(newX, newY), EmptyTile))
							{
								dig();
							}
						}
						nextMove = null;
					}
				}
				
				berryText.text = Std.string(score);
				
				var ripeEvilBerries:Int = 0;
				for (g in gridFeatures)
				{
					if (Std.is(g, Berry))
					{
						var b:Berry = cast(g, Berry);
						if (Std.is(getTile(b.gridX,b.gridY),EmptyTile) || getTile(b.gridX, b.gridY).evil)
						{
							gridFeatures.remove(b);
							var evilBerry:EvilBerry = new EvilBerry(b.gridX, b.gridY);
							gridFeatures.add(evilBerry);
						}
					}
				}
				
				for (g in gridFeatures)
				{
					if (Std.is(g, EvilBerry))
					{
						var b:EvilBerry = cast(g, EvilBerry);
						if (!getTile(b.gridX, b.gridY).evil)
						{
							gridFeatures.remove(b);
						}
						else
						{
							if (b.ripeness==3)
							{
								ripeEvilBerries++;
							}
						}
					}
				}
				evilBerryText.text = Std.string(ripeEvilBerries) + "/" + Std.string(EVIL_BERRY_LOSS_COUNT);
				
				for (g in gridFeatures)
				{
					if (Std.is(g, SeedPickup))
					{
						var s:SeedPickup = cast(g, SeedPickup);
						if (s.gridX == player.gridX && s.gridY == player.gridY)
						{
							gridFeatures.remove(s);
							pickupSeed();
						}
						else
						{
							if (!Std.is(getTile(s.gridX, s.gridY), Yellow))
							{
								gridFeatures.remove(s);
							}
						}
					}
				}
				
				if (FlxG.keys.anyJustPressed([SPACE]))
				{
					grow();
				}
				
				var rng:FlxRandom = new FlxRandom();
				
				if (frameCount % 4 == 0)
				{
					updateTiles();
				}
				
				seedText.text = Std.string(seedCount);
				
				var greenCount:Int = 0;
				for (t in tiles)
				{
					if (Std.is(t, Green))
					{
						greenCount++;
					}
				}
				for (g in gridFeatures)
				{
					if (Std.is(g, Berry))
					{
						greenCount--;
					}
				}
				
				berryDelay -= greenCount;
				if (berryDelay <= 0)
				{
					var potentialTiles:Array<Tile> = new Array<Tile>();
					for (t in tiles)
					{
						if (Std.is(t, Green))
						{
							var tileTaken:Bool = false;
							for (g in gridFeatures)
							{
								if (g.gridX == t.gridX && g.gridY == t.gridY)
								{
									tileTaken = true;
									break;
								}
							}
							if (!tileTaken)
							{
								potentialTiles.push(t);
							}
						}
					}
					
					if (potentialTiles.length > 0)
					{
						var chosenTile:Tile = potentialTiles[rng.int(0, potentialTiles.length - 1)];
						gridFeatures.add(new Berry(chosenTile.gridX, chosenTile.gridY));
					}
					berryDelay += BERRY_SPAWN_DELAY;
				}
				
				var evilCount:Int = 0;
				for (t in tiles)
				{
					if (t.evil)
					{
						evilCount++;
					}
				}
				for (g in gridFeatures)
				{
					if (Std.is(g, EvilBerry))
					{
						evilCount--;
					}
				}
				evilBerryDelay -= evilCount;
				if (evilBerryDelay <= 0)
				{
					var potentialTiles:Array<Tile> = new Array<Tile>();
					for (t in tiles)
					{
						if (t.evil)
						{
							var tileTaken:Bool = false;
							for (g in gridFeatures)
							{
								if (g.gridX == t.gridX && g.gridY == t.gridY)
								{
									tileTaken = true;
									break;
								}
							}
							if (!tileTaken)
							{
								potentialTiles.push(t);
							}
						}
					}
					
					if (potentialTiles.length > 0)
					{
						var chosenTile:Tile = potentialTiles[rng.int(0, potentialTiles.length - 1)];
						gridFeatures.add(new EvilBerry(chosenTile.gridX, chosenTile.gridY));
					}
					evilBerryDelay += EVIL_BERRY_SPAWN_DELAY;
				}
				
				var yellowCount:Int = 0;
				for (t in tiles)
				{
					if (Std.is(t, Yellow))
					{
						yellowCount++;
					}
				}
				for (g in gridFeatures)
				{
					if (Std.is(g, SeedPickup))
					{
						yellowCount--;
					}
				}
				
				seedDelay -= yellowCount;
				if (seedDelay <= 0)
				{
					var potentialTiles:Array<Tile> = new Array<Tile>();
					for (t in tiles)
					{
						if (Std.is(t, Yellow))
						{
							var tileTaken:Bool = false;
							for (g in gridFeatures)
							{
								if (g.gridX == t.gridX && g.gridY == t.gridY)
								{
									tileTaken = true;
									break;
								}
							}
							if (!tileTaken)
							{
								potentialTiles.push(t);
							}
						}
					}
					
					if (potentialTiles.length > 0)
					{
						var chosenTile:Tile = potentialTiles[rng.int(0, potentialTiles.length - 1)];
						gridFeatures.add(new SeedPickup(chosenTile.gridX, chosenTile.gridY));
					}
					seedDelay += SEED_SPAWN_DELAY;
				}
				
				if (ripeEvilBerries == EVIL_BERRY_LOSS_COUNT && gameEndTime ==-1)
				{
					var overlay:FlxSprite = new FlxSprite(0, 0);
					overlay.makeGraphic(FlxG.width, FlxG.height, 0x44000000);
					add(overlay);
					var lossText:FlxText = new FlxText(0, 32, FlxG.width, "Game Over", 16);
					lossText.alignment = CENTER;
					add(lossText);
					finalScore = score;
					var scoreText:FlxText = new FlxText(0, 64, FlxG.width, "Final Score: " + Std.string(score), 16);
					scoreText.alignment = CENTER;
					add(scoreText);
					
					gameEndTime = 120;
				}
				
				if (gameEndTime > 0)
				{
					gameEndTime--;
					if (gameEndTime == 0)
					{
						state = MENU;
						FlxG.switchState(new PlayState());
					}
				}
				
				updateMovementMap();
			case TUTORIAL:
				frameCount++;
				if (moveDelay > 0)
				{
					moveDelay--;
				}
				if (moveDelay <= 0)
				{
					if (FlxG.keys.anyPressed(["W", "w", UP]))
					{
						nextMove = new FlxPoint(0, -1);
					}
					if (FlxG.keys.anyPressed(["S", "s", DOWN]))
					{
						nextMove = new FlxPoint(0, 1);
					}
					if (FlxG.keys.anyPressed(["A", "a", LEFT]))
					{
						nextMove = new FlxPoint(-1, 0);
					}
					if (FlxG.keys.anyPressed(["D", "d", RIGHT]))
					{
						nextMove = new FlxPoint(1, 0);
					}
					if (nextMove != null)
					{
						var moveX:Int = Std.int(nextMove.x);
						var moveY:Int = Std.int(nextMove.y);
						var newX:Int = player.gridX + moveX;
						var newY:Int = player.gridY + moveY;
						if (newX >= 0 && newY >= 0 && newX < GRID_WIDTH && newY < GRID_HEIGHT && (movementMap.getTile(newX, newY) == 1 || getTile(newX, newY).walkable))
						{
							player.setGridPosition(newX, newY);
							moveDelay = MOVE_DELAY;
							for (g in gridFeatures)
							{
								if (Std.is(g, Berry))
								{
									var b:Berry = cast(g, Berry);
									if (b.gridX == player.gridX && b.gridY == player.gridY)
									{
										gridFeatures.remove(b);
										if (b.ripeness == 3)
										{
											score++;
										}
									}
								}
								if (Std.is(g, SeedPickup) && g.gridX == player.gridX && g.gridY == player.gridY)
								{
									pickupSeed();
									gridFeatures.remove(g);
								}
							}
							if (!Std.is(getTile(newX, newY), EmptyTile))
							{
								dig();
							}
						}
						nextMove = null;
					}
				}
				
				if (frameCount % 4 == 0)
				{
					updateTiles();
				}
				
				updateMovementMap();
				
				if (player.gridX >= GRID_WIDTH - 2 && player.gridY >= 7 && player.gridY < 9)
				{
					tutorialProgress++;
					if (tutorialProgress == 4)
					{
						state = MENU;
						tutorialProgress = 0;
						FlxG.switchState(new PlayState());
					}
					else
					{
						FlxG.switchState(new PlayState());
					}
				}
				
				if (FlxG.keys.anyJustPressed([SPACE]))
				{
					if (tutorialProgress < 2)
					{
						seedCount = 1000;
					}
					grow();
				}
				
				berryText.text = Std.string(score);
				seedText.text = Std.string(seedCount);
				
			case MENU:
				if (moveDelay > 0)
				{
					moveDelay--;
				}
				if (moveDelay <= 0)
				{
					if (FlxG.keys.anyPressed(["W", "w", UP]))
					{
						nextMove = new FlxPoint(0, -1);
					}
					if (FlxG.keys.anyPressed(["S", "s", DOWN]))
					{
						nextMove = new FlxPoint(0, 1);
					}
					if (FlxG.keys.anyPressed(["A", "a", LEFT]))
					{
						nextMove = new FlxPoint(-1, 0);
					}
					if (FlxG.keys.anyPressed(["D", "d", RIGHT]))
					{
						nextMove = new FlxPoint(1, 0);
					}
					if (nextMove != null)
					{
						var moveX:Int = Std.int(nextMove.x);
						var moveY:Int = Std.int(nextMove.y);
						var newX:Int = player.gridX + moveX;
						var newY:Int = player.gridY + moveY;
						if (newX >= 0 && newY >= 9 && newX < GRID_WIDTH && newY < 13)
						{
							player.setGridPosition(newX, newY);
							moveDelay = MOVE_DELAY;
						}
						nextMove = null;
					}
				}
				
				if (player.gridX < 2 && player.gridY >= 10 && player.gridY < 12)
				{
					state = PLAY;
					FlxG.switchState(new PlayState());
				}
				
				if (player.gridX >= GRID_WIDTH - 2 && player.gridY >= 10 && player.gridY < 12)
				{
					state = TUTORIAL;
					FlxG.switchState(new PlayState());
				}
		}
	}
	
	public function setTile(tile:Tile)
	{
		tiles.remove(grid[tile.gridX][tile.gridY]);
		grid[tile.gridX][tile.gridY] = tile;
		tiles.add(tile);
	}
	
	public static function inBounds(x:Int, y:Int):Bool
	{
		return x >= 0 && y >= 0 && x < GRID_WIDTH && y < GRID_HEIGHT;
	}
	
	public function getTile(x:Int, y:Int):Tile
	{
		if (inBounds(x, y))
		{
			return grid[x][y];
		}
		else
		{
			return null;
		}
	}
	
	public function grow()
	{
		if (seedCount > 0)
		{
			seedCount--;
			var seed:Seed = seedArray.shift();
			seeds.remove(seed);
			seed.place(player.gridX, player.gridY);
			
			for (s in seedArray)
			{
				s.moveUp();
			}
			
			if (seedCount > 3)
			{
				newSeed();
			}
		}
	}
	
	public function dig()
	{
		setTile(new EmptyTile(player.gridX, player.gridY));
		moveDelay = DESTROY_DELAY;
	}
	
	public function newSeed()
	{
		var index:Int = seedArray.length;
		var seed:Seed = null;
		switch(state)
		{
			case PLAY:
				switch(rng.int(0, 2))
				{
					case 0:
						seed = new GreenSeed(index, 1);
					case 1:
						seed = new YellowSeed(index, 1);
					case 2:
						seed = new OrangeSeed(index, 1);
				}
			case TUTORIAL:
				switch(tutorialProgress)
				{
					case 0:
						seed = new GreenSeed(index, 1);
					case 1:
						seed = new YellowSeed(index, 1);
					case 2:
						switch(rng.int(0, 1))
						{
							case 0:
								seed = new GreenSeed(index, 1);
							case 1:
								seed = new YellowSeed(index, 1);
						}
				}
			case MENU:
				
		}
		seeds.add(seed);
		seedArray.push(seed);
	}
	
	public function updateMovementMap()
	{
		for (i in 0...GRID_WIDTH)
		{
			for (i2 in 0...GRID_HEIGHT)
			{
				if (Std.is(getTile(i, i2), EmptyTile))
				{
					var adjacent:Bool = false;
					for (i3 in -1...2)
					{
						for (i4 in -1...2)
						{
							if (i + i3 >= 0 && i + i3 < GRID_WIDTH && i2 + i4 >= 0 && i2 + i4 < GRID_HEIGHT)
							{
								var tile:Tile = getTile(i + i3, i2 + i4);
								if (tile.adjacentWalkable)
								{
									adjacent = true;
								}
							}
						}
					}
					if (adjacent)
					{
						movementMap.setTile(i, i2, 1);
					}
					else
					{
						movementMap.setTile(i, i2, 0);
					}
				}
				else
				{
					movementMap.setTile(i, i2, 0);
				}
			}
		}
	}
	
	public function pickupSeed()
	{
		seedCount++;
		if (seedCount < 5)
		{
			newSeed();
		}
	}
	
	function spawnEvilPlant()
	{
		if (rng.float() < 0.6)
		{
			var potentialTiles:Array<Tile> = new Array<Tile>();
			for (t in tiles)
			{
				if (Std.is(t, EmptyTile) && (t.gridX == 0 || t.gridY == 0 || t.gridX == GRID_WIDTH - 1 || t.gridY == GRID_HEIGHT - 1))
				{
					potentialTiles.push(t);
				}
				if (Std.is(t, Red) && cast(t, Red).expand == 0)
				{
					potentialTiles.push(t);
				}
			}
			
			var chosenTile:Tile = potentialTiles[rng.int(0, potentialTiles.length - 1)];
			setTile(new Red(chosenTile.gridX, chosenTile.gridY, 1));
		}
		else
		{
			var x:Int = 0;
			var y:Int = 0;
			var direction:FlxPoint = new FlxPoint(0, 0);
			if (rng.float() < 0.5)
			{
				x = rng.int(3, GRID_WIDTH - 4);
				if (rng.float() < 0.5)
				{
					y = 0;
					direction.y = 1;
				}
				else
				{
					y = GRID_HEIGHT - 1;
					direction.y =-1;
				}
			}
			else
			{
				y = rng.int(3, GRID_WIDTH - 4);
				if (rng.float() < 0.5)
				{
					x = 0;
					direction.x = 1;
				}
				else
				{
					x = GRID_HEIGHT - 1;
					direction.x =-1;
				}
			}
			setTile(new Purple(x, y, 1, direction));
		}
	}
	
	public function updateTiles()
	{
		var tileArray:Array<Tile> = new Array<Tile>();
		for (i in 0...GRID_WIDTH)
		{
			for (i2 in 0...GRID_HEIGHT)
			{
				tileArray.insert(rng.int(0, tileArray.length - 1), getTile(i, i2));
			}
		}
		
		for (t in tileArray)
		{
			if (t.needsUpdate)
			{
				t.tileUpdate();
			}
		}
	}
}