package tiles;
import flixel.FlxG;
import flixel.math.FlxRandom;

/**
 * ...
 * @author Ethan Hoeppner
 */
class Green extends Tile
{
	public static inline var EXPAND_MAX:Int = 13;
	public static inline var EXPAND_MIN:Int = 10;
	public static inline var PROGRESS_MAX:Int = 11;
	public static inline var PROGRESS_MIN:Int = 6;
	
	public var expand:Int;
	
	var upProgress:Int;
	var upProgressMax:Int;
	var downProgress:Int;
	var downProgressMax:Int;
	var leftProgress:Int;
	var leftProgressMax:Int;
	var rightProgress:Int;
	var rightProgressMax:Int;
	
	var directionOrders:Array<Int>;
	var directionIndex:Int;
	
	public function new(x:Int, y:Int, rank:Int, ?expand:Int) 
	{
		super(x, y, rank);
		adjacentWalkable = true;
		makeGraphic(8, 8, 0xff00ff00);
		if (expand == null)
		{
			this.expand = rng.int(EXPAND_MIN, EXPAND_MAX);
		}
		else
		{
			this.expand = expand;
		}
		
		upProgress = 0;
		downProgress = 0;
		leftProgress = 0;
		rightProgress = 0;
		upProgressMax = rng.int(PROGRESS_MIN, PROGRESS_MAX);
		downProgressMax = rng.int(PROGRESS_MIN, PROGRESS_MAX);
		leftProgressMax = rng.int(PROGRESS_MIN, PROGRESS_MAX);
		rightProgressMax = rng.int(PROGRESS_MIN, PROGRESS_MAX);
		
		directionIndex = 0;
		directionOrders = new Array<Int>();
		for (i in 0...4)
		{
			directionOrders.insert(rng.int(0, i), i);
		}
		
		needsUpdate = true;
		tileType = GREEN;
	}
	
	override public function tileUpdate()
	{
		if (expand > 0)
		{
			directionIndex++;
			if (directionIndex >= 4)
			{
				directionIndex = 0;
			}
			switch(directionOrders[directionIndex])
			{
				case 0:
					if (PlayState.inBounds(gridX, gridY - 1))
					{
						if (PlayState.playState.getTile(gridX, gridY - 1).rank < rank)
						{
							upProgress++;
						}
						else
						{
							upProgress = 0;
						}
						if (upProgress == upProgressMax)
						{
							expand--;
							var splitExpand:Int = rng.int(0, expand);
							expand -= splitExpand;
							PlayState.playState.setTile(new Green(gridX, gridY - 1, rank, splitExpand));
						}
					}
				case 1:
					if (PlayState.inBounds(gridX, gridY + 1))
					{
						if (PlayState.playState.getTile(gridX, gridY + 1).rank < rank)
						{
							downProgress++;
						}
						else
						{
							downProgress = 0;
						}
						if (downProgress == downProgressMax)
						{
							expand--;
							var splitExpand:Int = rng.int(0, expand);
							expand -= splitExpand;
							PlayState.playState.setTile(new Green(gridX, gridY + 1, rank, splitExpand));
						}
					}
				case 2:
					if (PlayState.inBounds(gridX - 1, gridY))
					{
						if (PlayState.playState.getTile(gridX - 1, gridY).rank < rank)
						{
							leftProgress++;
						}
						else
						{
							leftProgress = 0;
						}
						if (leftProgress == leftProgressMax)
						{
							expand--;
							var splitExpand:Int = rng.int(0, expand);
							expand -= splitExpand;
							PlayState.playState.setTile(new Green(gridX - 1, gridY, rank, splitExpand));
						}
					}
				case 3:
					if (PlayState.inBounds(gridX + 1, gridY))
					{
						if (PlayState.playState.getTile(gridX + 1, gridY).rank < rank)
						{
							rightProgress++;
						}
						else
						{
							rightProgress = 0;
						}
						if (rightProgress == rightProgressMax)
						{
							expand--;
							var splitExpand:Int = rng.int(0, expand);
							expand -= splitExpand;
							PlayState.playState.setTile(new Green(gridX + 1, gridY, rank, splitExpand));
						}
					}
			}
		}
		if (expand == 0)
		{
			color = 0x888888;
			needsUpdate = false;
		}
		super.tileUpdate();
	}
}