package tiles;

import tiles.Tile;

/**
 * ...
 * @author Ethan Hoeppner
 */
class Red extends Tile
{
	public static inline var EXPAND:Int = 3;
	
	public static inline var PROGRESS_MAX:Int = 40;
	public static inline var PROGRESS_MIN:Int = 25;
	public static inline var PROGRESS_FACTOR:Int = 4;
	
	var upProgress:Int;
	var upProgressMax:Int;
	var downProgress:Int;
	var downProgressMax:Int;
	var leftProgress:Int;
	var leftProgressMax:Int;
	var rightProgress:Int;
	var rightProgressMax:Int;
	
	var directionOrders:Array<Int>;
	var directionIndex:Int;
	
	public var expand:Int;
	
	public function new(x:Int, y:Int, rank:Int, ?expand:Int) 
	{
		super(x, y, rank);
		
		if (expand == null)
		{
			expand = EXPAND;
		}
		
		this.expand = expand;
		
		switch(expand)
		{
			case 0:
				makeGraphic(8, 8, 0xff440000);
			case 1:
				makeGraphic(8, 8, 0xff880000);
			case 2:
				makeGraphic(8, 8, 0xffcc0000);
			case 3:
				makeGraphic(8, 8, 0xffff0000);
		}
		
		upProgress = 0;
		downProgress = 0;
		leftProgress = 0;
		rightProgress = 0;
		upProgressMax = rng.int(PROGRESS_MIN, PROGRESS_MAX);
		downProgressMax = rng.int(PROGRESS_MIN, PROGRESS_MAX);
		leftProgressMax = rng.int(PROGRESS_MIN, PROGRESS_MAX);
		rightProgressMax = rng.int(PROGRESS_MIN, PROGRESS_MAX);
		
		directionIndex = 0;
		directionOrders = new Array<Int>();
		for (i in 0...4)
		{
			directionOrders.insert(rng.int(0, i), i);
		}
		
		needsUpdate = true;
		tileType = RED;
		
		evil = true;
	}
	
	override public function tileUpdate()
	{
		directionIndex++;
		if (directionIndex >= 4)
		{
			directionIndex = 0;
		}
		switch(directionOrders[directionIndex])
		{
			case 0:
				if (PlayState.inBounds(gridX, gridY - 1) && !Std.is(PlayState.playState.getTile(gridX, gridY - 1), Red))
				{
					var upTile:Tile = PlayState.playState.getTile(gridX, gridY - 1);
					if (upTile.rank <= rank)
					{
						if (Std.is(upTile, EmptyTile))
						{
							if (expand > 0)
							{
								upProgress++;
							}
						}
						else
						{
							upProgress += PROGRESS_FACTOR;
						}
					}
					else
					{
						upProgress = 0;
					}
					if (upProgress >= upProgressMax)
					{
						PlayState.playState.setTile(new Red(gridX, gridY - 1, rank, Std.int(Math.max(expand - 1, 0))));
					}
				}
				else
				{
					upProgress = 0;
				}
			case 1:
				if (PlayState.inBounds(gridX, gridY + 1) && !Std.is(PlayState.playState.getTile(gridX, gridY + 1), Red))
				{
					var downTile:Tile = PlayState.playState.getTile(gridX, gridY + 1);
					if (downTile.rank <= rank)
					{
						if (Std.is(downTile, EmptyTile))
						{
							if (expand > 0)
							{
								downProgress++;
							}
						}
						else
						{
							downProgress += PROGRESS_FACTOR;
						}
					}
					else
					{
						downProgress = 0;
					}
					if (downProgress >= downProgressMax)
					{
						PlayState.playState.setTile(new Red(gridX, gridY + 1, rank, Std.int(Math.max(expand - 1, 0))));
					}
				}
				else
				{
					downProgress = 0;
				}
			case 2:
				if (PlayState.inBounds(gridX - 1, gridY) && !Std.is(PlayState.playState.getTile(gridX - 1, gridY), Red))
				{
					var leftTile:Tile = PlayState.playState.getTile(gridX - 1, gridY);
					if (leftTile.rank <= rank)
					{
						if (Std.is(leftTile, EmptyTile))
						{
							if (expand > 0)
							{
								leftProgress++;
							}
						}
						else
						{
							leftProgress += PROGRESS_FACTOR;
						}
					}
					else
					{
						leftProgress = 0;
					}
					if (leftProgress >= leftProgressMax)
					{
						PlayState.playState.setTile(new Red(gridX - 1, gridY, rank, Std.int(Math.max(expand - 1, 0))));
					}
				}
				else
				{
					leftProgress = 0;
				}
			case 3:
				if (PlayState.inBounds(gridX + 1, gridY) && !Std.is(PlayState.playState.getTile(gridX + 1, gridY), Red))
				{
					var rightTile:Tile = PlayState.playState.getTile(gridX + 1, gridY);
					if (rightTile.rank <= rank)
					{
						if (Std.is(rightTile, EmptyTile))
						{
							if (expand > 0)
							{
								rightProgress++;
							}
						}
						else
						{
							rightProgress += PROGRESS_FACTOR;
						}
					}
					else
					{
						rightProgress = 0;
					}
					if (rightProgress >= rightProgressMax)
					{
						PlayState.playState.setTile(new Red(gridX + 1, gridY, rank, Std.int(Math.max(expand - 1, 0))));
					}
				}
				else
				{
					rightProgress = 0;
				}
		}
		
		super.tileUpdate();
	}
}