package tiles;

import flixel.FlxG;
import flixel.math.FlxPoint;
import flixel.math.FlxRandom;
import tiles.Tile;

/**
 * ...
 * @author Ethan Hoeppner
 */
class Yellow extends Tile
{
	public static inline var MAX_EXPAND:Int = 6;
	public static inline var MIN_EXPAND:Int = 4;
	public static inline var PROGRESS:Int = 7;
	
	public var expand:Int;
	
	public var direction:FlxPoint;
	
	public var progress:Int;
	
	public function new(x:Int, y:Int, rank:Int, ?expand:Int, ?direction:FlxPoint) 
	{
		super(x, y, rank);
		adjacentWalkable = true;
		makeGraphic(8, 8, 0xffffff00);
		needsUpdate = true;
		progress = 0;
		if (expand == null)
		{
			this.expand = rng.int(MIN_EXPAND, MAX_EXPAND);
		}
		else
		{
			this.expand = expand;
		}
		
		if (direction == null)
		{
			direction = new FlxPoint(0,0);
			if (rng.float() < 0.5)
			{
				if (rng.float() < 0.5)
				{
					direction.x = 1;
				}
				else
				{
					direction.x = -1;
				}
			}
			else
			{
				if (rng.float() < 0.5)
				{
					direction.y = 1;
				}
				else
				{
					direction.y = -1;
				}
			}
		}
		this.direction = direction;
		tileType = YELLOW;
	}
	
	override public function tileUpdate()
	{
		if (expand > 0)
		{
			progress++;
			if (progress >= PROGRESS)
			{
				if (!Std.is(PlayState.playState.getTile(gridX + Std.int(direction.x), gridY + Std.int(direction.y)), EmptyTile))
				{
					var potentialDirections:Array<FlxPoint> = new Array<FlxPoint>();
					if (Std.is(PlayState.playState.getTile(gridX, gridY + 1), EmptyTile))
					{
						potentialDirections.push(new FlxPoint(0, 1));
					}
					if (Std.is(PlayState.playState.getTile(gridX, gridY - 1), EmptyTile))
					{
						potentialDirections.push(new FlxPoint(0, -1));
					}
					if (Std.is(PlayState.playState.getTile(gridX + 1, gridY), EmptyTile))
					{
						potentialDirections.push(new FlxPoint(1, 0));
					}
					if (Std.is(PlayState.playState.getTile(gridX - 1, gridY), EmptyTile))
					{
						potentialDirections.push(new FlxPoint(-1, 0));
					}
					if (potentialDirections.length > 0)
					{
						direction = potentialDirections[rng.int(0, potentialDirections.length - 1)];
					}
				}
				if (Std.is(PlayState.playState.getTile(gridX + Std.int(direction.x), gridY + Std.int(direction.y)), EmptyTile))
				{
					needsUpdate = false;
					PlayState.playState.setTile(new Yellow(gridX + Std.int(direction.x), gridY + Std.int(direction.y), rank, expand - 1, direction));
				}
			}
		}
	}
}